==> nnn-2024.2.17.tgz: package/jsOnParse.js <==
import { has } from './has.js'
import { is } from './is.js'

/**
 * `JSON.parse` with “JavaScript turned on”.
 *
 * Objects having *exactly* one property which is present in the `handlers` map, i.e. objects of the form:
 *
 * ```js
 * { "«handlerName»": [«params»] }
 * ```
 *
 * are replaced by the result of call
 *
 * ```js
 * handlers['«handlerName»'](...«params»)
 * ```
 */
export const jsOnParse = (
  /** @type {Partial<Record<PropertyKey, Function>>} */ handlers,
  /** @type {string}                   */ text
) => JSON.parse(text, (key, value) => {
  if (is(Object, value)) {
    let isSecondKey = false

    for (key in value) {
      if (isSecondKey) {
        return value
      }
      isSecondKey = true
    }

    if (has(key, handlers) && is(Array, value[key])) {
      // @ts-expect-error
      return handlers[key](...value[key])
    }
  }

  return value
})

export const tests = {
  jsOnParse: (/** @type {import('bun:test').Expect} */ expect) => {
    const handlers = {
      $hello: (/** @type {string} */ name) => `Hello ${name}!`,
      $foo: () => 'bar'
    }
    const actual = jsOnParse(handlers, `[
      {
        "$hello": ["World"]
      },
      {
        "nested": {
          "$hello": ["nested World"]
        },
        "one": 1,
        "two": 2
      },
      {
        "$foo": []
      },
      {
        "$foo": ["The parent object does not have exactly one property!"],
        "one": 1,
        "two": 2
      }
    ]`)
    const expected = [
      'Hello World!',
      {
        nested: 'Hello nested World!',
        one: 1,
        two: 2
      },
      'bar',
      {
        $foo: ['The parent object does not have exactly one property!'],
        one: 1,
        two: 2
      }
    ]

    expect(actual).toEqual(expected)
  }
}

==> nnn-2024.2.20.tgz: package/jsOnParse.js <==
import { has } from './has.js'
import { is } from './is.js'

/**
 * `JSON.parse` with “JavaScript turned on”.
 *
 * Objects having *exactly* one property which is present in the `handlers` map, i.e. objects of the form:
 *
 * ```js
 * { "«handlerName»": [«params»] }
 * ```
 *
 * are replaced by the result of call
 *
 * ```js
 * handlers['«handlerName»'](...«params»)
 * ```
 */
export const jsOnParse = (
  /** @type {Partial<Record<PropertyKey, Function>>} */ handlers,
  /** @type {string}                   */ text
) => JSON.parse(text, (key, value) => {
  if (is(Object, value)) {
    let isSecondKey = false

    for (key in value) {
      if (isSecondKey) {
        return value
      }
      isSecondKey = true
    }

    if (has(key, handlers) && is(Array, value[key])) {
      // @ts-expect-error
      return handlers[key](...value[key])
    }
  }

  return value
})

export const tests = {
  jsOnParse: (/** @type {import('chai').expect} */ expect) => {
    const handlers = {
      $hello: (/** @type {string} */ name) => `Hello ${name}!`,
      $foo: () => 'bar'
    }
    const actual = jsOnParse(handlers, `[
      {
        "$hello": ["World"]
      },
      {
        "nested": {
          "$hello": ["nested World"]
        },
        "one": 1,
        "two": 2
      },
      {
        "$foo": []
      },
      {
        "$foo": ["The parent object does not have exactly one property!"],
        "one": 1,
        "two": 2
      }
    ]`)
    const expected = [
      'Hello World!',
      {
        nested: 'Hello nested World!',
        one: 1,
        two: 2
      },
      'bar',
      {
        $foo: ['The parent object does not have exactly one property!'],
        one: 1,
        two: 2
      }
    ]

    expect(actual).to.deep.equal(expected)
  }
}
